package com.sun.corba.se.impl.protocol;

import com.sun.corba.se.impl.orbutil.ORBUtility;
import org.omg.CORBA.portable.RemarshalException;
import org.omg.CORBA.portable.ApplicationException;
import com.sun.corba.se.impl.encoding.ByteBufferWithInfo;
import java.io.IOException;
import java.security.AccessController;
import com.sun.corba.se.impl.protocol.giopmsgheaders.Message;
import java.nio.ByteBuffer;
import java.security.PrivilegedAction;
import com.sun.corba.se.impl.encoding.CDRInputObject;
import com.sun.corba.se.impl.encoding.CDROutputObject;
import com.sun.corba.se.spi.orb.ORB;
import com.sun.corba.se.spi.protocol.CorbaMessageMediator;
import com.sun.corba.se.pept.encoding.InputObject;
import com.sun.corba.se.pept.encoding.OutputObject;

public class SharedCDRClientRequestDispatcherImpl extends CorbaClientRequestDispatcherImpl
{
    public SharedCDRClientRequestDispatcherImpl() {
    }
    
    @Override
    public InputObject marshalingComplete(Object o, OutputObject outputObject) throws ApplicationException, RemarshalException {
        final ORB orb = null;
        CorbaMessageMediator corbaMessageMediator = null;
        try {
            corbaMessageMediator = (CorbaMessageMediator)outputObject.getMessageMediator();
            orb = (ORB)corbaMessageMediator.getBroker();
            if (orb.subcontractDebugFlag) {
                this.dprint(".marshalingComplete->: " + this.opAndId(corbaMessageMediator));
            }
            CDROutputObject cdrOutputObject = (CDROutputObject)outputObject;
            ByteBufferWithInfo byteBufferWithInfo = cdrOutputObject.getByteBufferWithInfo();
            cdrOutputObject.getMessageHeader().setSize(byteBufferWithInfo.byteBuffer, byteBufferWithInfo.getSize());
            CDRInputObject inputObject = (CDRInputObject)AccessController.doPrivileged(new PrivilegedAction<CDRInputObject>() {
                
                @Override
                public CDRInputObject run() {
                    return new CDRInputObject(orb, null, inBuffer, inMsg);
                }
            });
            corbaMessageMediator.setInputObject(inputObject);
            inputObject.setMessageMediator(corbaMessageMediator);
            ((CorbaMessageMediatorImpl)corbaMessageMediator).handleRequestRequest(corbaMessageMediator);
            try {
                inputObject.close();
            }
            catch (IOException ex) {
                if (orb.transportDebugFlag) {
                    this.dprint(".marshalingComplete: ignoring IOException - " + ex.toString());
                }
            }
            CDROutputObject cdrOutputObject2 = (CDROutputObject)corbaMessageMediator.getOutputObject();
            ByteBufferWithInfo byteBufferWithInfo2 = cdrOutputObject2.getByteBufferWithInfo();
            cdrOutputObject2.getMessageHeader().setSize(byteBufferWithInfo2.byteBuffer, byteBufferWithInfo2.getSize());
            CDRInputObject inputObject2 = (CDRInputObject)AccessController.doPrivileged(new PrivilegedAction<CDRInputObject>() {
                
                @Override
                public CDRInputObject run() {
                    return new CDRInputObject(orb, null, inBuffer2, inMsg2);
                }
            });
            corbaMessageMediator.setInputObject(inputObject2);
            inputObject2.setMessageMediator(corbaMessageMediator);
            inputObject2.unmarshalHeader();
            InputObject processResponse = this.processResponse(orb, corbaMessageMediator, inputObject2);
        }
        finally {
            if (orb.subcontractDebugFlag) {
                this.dprint(".marshalingComplete<-: " + this.opAndId(corbaMessageMediator));
            }
        }
    }
    
    @Override
    protected void dprint(String s) {
        ORBUtility.dprint("SharedCDRClientRequestDispatcherImpl", s);
    }
}
